## $Id: browser.py 8421 2012-05-11 15:46:29Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""UI components for basic applicants and related components.
"""
import grok
from zope.component import getUtility
from zope.i18n import translate
from waeup.kofa.widgets.datewidget import FriendlyDatetimeDisplayWidget
from waeup.kofa.students.interfaces import IStudentsUtils
from waeup.kofa.applicants.interfaces import (
    IApplicantRegisterUpdate, IApplicant, IApplicantEdit)
from waeup.kofa.applicants.browser import (ApplicantDisplayFormPage,
    ExportPDFPage,
    ApplicantManageFormPage, ApplicantEditFormPage,
    ApplicantRegistrationPage, ApplicantAddFormPage,
    OnlinePaymentDisplayFormPage, ApplicationFeePaymentAddPage,
    OnlinePaymentBreadcrumb, ExportPDFPaymentSlipPage,
    ApplicantBaseDisplayFormPage)
from waeup.kofa.applicants.viewlets import (
    PaymentReceiptActionButton)
from waeup.kofa.applicants.pdf import PDFApplicationSlip
from waeup.uniben.applicants.interfaces import (
    IPGApplicant, IUGApplicant, IPGApplicantEdit, IUGApplicantEdit,
    ICustomApplicantOnlinePayment)
from waeup.uniben.interfaces import MessageFactory as _

#class RequestCallbackActionButton(RequestCallbackActionButton):
#    """ Display the base package callback button in custom pages.
#    """
#    grok.context(ICustomApplicantOnlinePayment)

#class CustomOnlinePaymentCallbackPage(OnlinePaymentCallbackPage):
#    """ Activate callback simulation view
#    """
#    grok.context(ICustomApplicantOnlinePayment)

class CustomOnlinePaymentBreadcrumb(OnlinePaymentBreadcrumb):
    """A breadcrumb for payments.
    """
    grok.context(ICustomApplicantOnlinePayment)

class PaymentReceiptActionButton(PaymentReceiptActionButton):
    grok.order(4)
    grok.context(ICustomApplicantOnlinePayment)

class CustomPDFApplicationSlip(PDFApplicationSlip):

    @property
    def form_fields(self):
        target = getattr(self.context.__parent__, 'prefix', None)
        if target is not None and target.startswith('pg'):
            form_fields = grok.AutoFields(IPGApplicant).omit(
                'locked', 'course_admitted')
        else:
            form_fields = grok.AutoFields(IUGApplicant).omit(
                'locked', 'course_admitted')
        return form_fields

class CustomApplicantDisplayFormPage(ApplicantDisplayFormPage):
    """A display view for applicant data.
    """

    @property
    def form_fields(self):
        target = getattr(self.context.__parent__, 'prefix', None)
        if target is not None and target.startswith('pg'):
            form_fields = grok.AutoFields(IPGApplicant).omit(
                'locked', 'course_admitted', 'password')
        else:
            form_fields = grok.AutoFields(IUGApplicant).omit(
                'locked', 'course_admitted', 'password')
        return form_fields

class CustomApplicantManageFormPage(ApplicantManageFormPage):
    """A full edit view for applicant data.
    """
    
    @property
    def form_fields(self):
        target = getattr(self.context.__parent__, 'prefix', None)
        if target is not None and target.startswith('pg'):
            form_fields = grok.AutoFields(IPGApplicant)
        else:
            form_fields = grok.AutoFields(IUGApplicant)
        form_fields['student_id'].for_display = True
        form_fields['applicant_id'].for_display = True
        return form_fields

class CustomApplicantEditFormPage(ApplicantEditFormPage):
    """An applicant-centered edit view for applicant data.
    """

    @property
    def form_fields(self):
        target = getattr(self.context.__parent__, 'prefix', None)
        if target is not None and target.startswith('pg'):
            form_fields = grok.AutoFields(IPGApplicantEdit).omit(
                'locked', 'course_admitted', 'student_id',
                'screening_score', 'screening_venue'
                )
        else:
            form_fields = grok.AutoFields(IUGApplicantEdit).omit(
                'locked', 'course_admitted', 'student_id',
                'screening_score'
                )
        form_fields['applicant_id'].for_display = True
        form_fields['reg_number'].for_display = True
        return form_fields

class CustomOnlinePaymentDisplayFormPage(OnlinePaymentDisplayFormPage):
    """ Page to view an online payment ticket
    """
    grok.context(ICustomApplicantOnlinePayment)
    form_fields = grok.AutoFields(ICustomApplicantOnlinePayment).omit('ac')
    form_fields[
        'creation_date'].custom_widget = FriendlyDatetimeDisplayWidget('le')
    form_fields[
        'payment_date'].custom_widget = FriendlyDatetimeDisplayWidget('le')
    grok.template('payment_view')

    @property
    def transaction_code(self):
        tcode = self.context.p_id
        return tcode[len(tcode)-8:len(tcode)]

class CustomApplicationFeePaymentAddPage(ApplicationFeePaymentAddPage):
    """ Page to add an online payment ticket
    """
    factory = u'waeup.CustomApplicantOnlinePayment'

    def _fillCustomFields(self, payment, session_config):
        # No custom fields at the moment
        return payment

class CustomExportPDFPaymentSlipPage(ExportPDFPaymentSlipPage):
    """Deliver a PDF slip of the context.
    """
    grok.context(ICustomApplicantOnlinePayment)
    form_fields = grok.AutoFields(ICustomApplicantOnlinePayment).omit('ac')
    form_fields['creation_date'].custom_widget = FriendlyDatetimeDisplayWidget('le')
    form_fields['payment_date'].custom_widget = FriendlyDatetimeDisplayWidget('le')

    @property
    def note(self):
        if self.context.p_state == 'paid':
            return None
        tcode = self.context.p_id
        tcode = tcode[len(tcode)-8:len(tcode)]
        amount = self.context.amount_auth
        note = translate(_(
            u"""<br /><br /><br />
The tranzaction code for eTranzact payments is <strong>${a}</strong>.""",
            mapping = {'a':tcode}))
        return note
